#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

#define SUS 1
#define STG 2
#define DRT 3
#define JOS 4


/* Lungime si pozitie initiala*/
#define LUNG_INIT 3
#define POZ_X_INI 40
#define POZ_Y_INI 40

/*Moduri de gameplay*/
#define EASY 0
#define HARD 1

#define DIM 2


int scor = 0;
int mod;
int vieti = 3;

typedef struct {
	char x;
	char y;
} Block;

Block snake[100];
Block food;
int lungime;
char directie;



void desenMap(void) {

	
	int i = 0, j, k;
	char x1, x2, y1, y2;
	
    
	while(i < lungime) {
	
		x1 = snake[i].x; //setez sarpele pe LCD
		y1 = snake[i].y;
		x2 = snake[i].x + DIM;
		y2 = snake[i].y + DIM;
		
		for(j = x1; j < x2; j++) {
			for(k = y1; k < y2; k++) {
				lcd_setpixel(j, k);
			}
		}
       i++;
		
	}
	
	x1 = food.x; //setez punctul de mancare pe LCD
	y1 = food.y;
	x2 = food.x + DIM;
	y2 = food.y + DIM;
	
	for(j = x1; j < x2; j++) {
		for(k = y1; k < y2; k++) {
			lcd_setpixel(j, k);
		}
	}
		
	if (mod == HARD) { //setez peretii
	
        i = 0;
        while (i < LCD_Y_RES) {
               
            lcd_setpixel(0, i);
			lcd_setpixel(LCD_X_RES - 1, i);
            i++;   
        }
		
        i = 0;
		while (i < LCD_X_RES) {
            
            lcd_setpixel(i, 0);
			lcd_setpixel(i, LCD_Y_RES - 1);
            i++;
        }
	}
}


void init(void){

	int i;
	lungime = LUNG_INIT;
	directie = DRT;
	
	snake[0].x = POZ_X_INI;
	snake[0].y = POZ_Y_INI;
    
    i = 1;
	while(i < lungime){
    
		snake[i].x = snake[0].x - i * DIM; //plec spre dreapta, cresc in stanga
		snake[i].y = snake[0].y;
        i++;
	}
	food.x = rand()%(LCD_X_RES - DIM);
	food.y = rand()%(LCD_Y_RES - DIM);

}


void deplasare(char dir) {

	int i = lungime - 1;
	
	//Blocurile din coada urmaresc capatul
    
	while (i > 0) {
		snake[i].x = snake[i - 1].x; 
		snake[i].y = snake[i - 1].y;
        i--;
	}
	
	int poz_y = LCD_Y_RES - DIM;
    int poz_x = LCD_X_RES - DIM;
    
	if (dir == SUS) {
	
		if (snake[0].y <= 0) {
            snake[0].y = poz_y;
			
		} else {
			snake[0].y -= DIM;
		}
		
	} else if (dir == JOS) {
	
		if (snake[0].y >= poz_y) {
            snake[0].y = 0;
		} else {
			snake[0].y += DIM;
		}
		
	} else if (dir == STG) {
	
		if (snake[0].x <= 0) {
            snake[0].x = poz_x;
		} else {
			snake[0].x -= DIM;
		}
	
	} else if (dir == DRT) {
	
		if (snake[0].x >= poz_x) {
            snake[0].x = 0;
		} else {
			snake[0].x += DIM;
		}
	
	}
	
}



void SnakeDelay(void) {

	volatile int i;
	for (i = -32000; i < 32000; i++);

}



void mode(void) {
	

	lcd_clear();
	lcd_update();
	
    lcd_gotoXY(25, 0);
    lcd_str("Menu", 4);
	lcd_gotoXY(2, 1);
	lcd_str("select mode", 11);
	
	lcd_gotoXY(10, 3);
	if (mod == EASY)
		lcd_str("# Easy", 6);
	else
		lcd_str("  Easy", 6);
	
	lcd_gotoXY(10, 4);
	if (mod == HARD)
		lcd_str("# HARD", 6);
	else
		lcd_str("  HARD", 6);
	
	while(1) {
	
		
		// s-a selectat mod de joc
		if (!(PINA & (1 << PA0))) 
			break;
		
		// jos
		if (!(PINA & (1 << PA4))) {
			mod = (mod + 1) % 2;
			lcd_clear();
            lcd_update();
            
            lcd_gotoXY(25, 0);
            lcd_str("Menu", 4);
            lcd_gotoXY(2, 1);
            lcd_str("select mode", 11);
	
            lcd_gotoXY(10, 3);
            if (mod == EASY)
                lcd_str("# Easy", 6);
            else
                lcd_str("  Easy", 6);
        
            lcd_gotoXY(10, 4);
            if (mod == HARD)
                lcd_str("# HARD", 6);
            else
                lcd_str("  HARD", 6);
			
		}
		
		// sus
		if (!(PINA & (1 << PA3))) {
			mod = mod - 1;
			if (mod == -1)
				mod = 1;
			lcd_clear();
            lcd_update();
            
            lcd_gotoXY(25, 0);
            lcd_str("Menu", 4);
            lcd_gotoXY(2, 1);
            lcd_str("select mode", 11);
        
            lcd_gotoXY(10, 3);
            if (mod == EASY)
                lcd_str("# Easy", 6);
            else
                lcd_str("  Easy", 6);
	
            lcd_gotoXY(10, 4);
            if (mod == HARD)
                lcd_str("# HARD", 6);
            else
                lcd_str("  HARD", 6);
			
		}
		
		SnakeDelay(); 
		
	}
		
	scor = 0;
	SnakeDelay();
	lcd_clear();
	lcd_update();
	init();
	
}


void game_over(void) {

	char numar[2];
	
	itoa(scor, numar, 10);
	
	lcd_init();
	lcd_clear();
	lcd_update();
	
	lcd_gotoXY(12, 1);	
	lcd_str("Game Over", 9);
	
	lcd_gotoXY(16, 3);	
	lcd_str("SCOR:", 5);
	
	lcd_gotoXY(50, 3);
	if (scor >= 10) //iau pe cazuri, limitare de scor la 99, dimensiunea fiind max 100
		
        lcd_str(numar, 2);
	else 
		lcd_str(numar, 1);
		
	while(1){
	
		// reset
		if (!(PINA & (1 << PA0))) {
		
			mode();
			break;
		}
		
	}
	
}


void verifica(void) {

	int i;
		
	// sarpele a iesit din rezolutie (pentru HARD)
	if (mod == HARD) {
		if ((snake[0].x <= 0) || (snake[0].x >= LCD_X_RES))  //imi iese din aria axei Ox
                    game_over();
			else if ((snake[0].y <= 0) || (snake[0].y >= LCD_Y_RES)) 	//imi iese din aria axei Oy
                        game_over();
		
	}
    
    if (mod == EASY) {
        for(i = 1; i < lungime; i++){
		if ((snake[0].x == snake[i].x) && (snake[0].y == snake[i].y)){
			vieti--;
			if (vieti > 0) {
				
                char nr_vieti[1];
                itoa(vieti, nr_vieti, 10);
                lcd_gotoXY(12, 1);	
                lcd_str("Nr vieti:", 9);
                lcd_gotoXY(50, 3);
                lcd_str(nr_vieti, 1);
                _delay_ms(3000);
				init();
                }
			else {
				game_over();
             }
		}
	}
  }
    
	if (mod == HARD) { // accident :))
    i = 1;
	while (i < lungime) { //fara capul sarpelui
		if ((snake[0].x == snake[i].x) && (snake[0].y == snake[i].y)) { //daca fata sarpeului intra intr-un alt bloc din corp
			game_over();	
		}
       i++;
	}
}
	
	if (((snake[0].x <= food.x + DIM) && (snake[0].y <= food.y + DIM) && (snake[0].x >= food.x) && (snake[0].y >= food.y)) ||
		 ((snake[0].x + DIM <= food.x + DIM) && (snake[0].y + DIM <= food.y + DIM) && (snake[0].x + DIM >= food.x) && (snake[0].y + DIM >= food.y))) {
		
	
	
		// maresc dimensiunea
		snake[lungime].x = snake[lungime - 1].x;
		snake[lungime].y = snake[lungime - 1].y;
		lungime++;

		// mancare noua
		food.x = rand() % (LCD_X_RES - DIM);
		food.y = rand() % (LCD_Y_RES - DIM);
		
		
		// cresc cu 1 unitate
		scor++;
        
        
		
	}
	
}




int main(void) {

	
	lcd_init();
	lcd_clear();
	lcd_update();
	
	DDRA &= ~(1 << PA4); // jos
	PORTA |= (1 << PA4);
	
	DDRA &= ~(1 << PA2); // dreapta
	PORTA |= (1 << PA2);
	
	DDRA &= ~(1 << PA3); // sus
	PORTA |= (1 << PA3);
	
	DDRA &= ~(1 << PA1); // stanga
	PORTA |= (1 << PA1);
	
	DDRA &= ~(1 << PA0); // select
	PORTA |= (1 << PA0);
	
	DDRD |= (1 << PD5); // speaker
	

	mode();
	
    lcd_gotoXY(11, 3);
    lcd_str("Be ready", 8);
    _delay_ms(1000000);
	
    if (mod == HARD) {
        while(1) {
	
    
		lcd_clear();
		
        
        if (!(PINA & (1 << PA3)) && (directie != JOS))  directie = SUS;
        if (!(PINA & (1 << PA4)) && (directie != SUS))  directie = JOS;
        if (!(PINA & (1 << PA1)) && (directie != DRT))  directie = STG;
        if (!(PINA & (1 << PA2)) && (directie != STG))  directie = DRT;
		
	
		if (!(PINA & (1 << PA0))) { //resetez, revin la meniu
			
			mode();
            lcd_gotoXY(11, 3);
            lcd_str("Be ready", 8);
			continue;
		}
		
			
		deplasare(directie);
		verifica();
		desenMap();
		SnakeDelay();
		lcd_update();
        }
		
	}
    else if (mod == EASY) {
            while (vieti > 0) {
            
                lcd_clear();
		
        
                if (!(PINA & (1 << PA3)) && (directie != JOS))  directie = SUS;
                if (!(PINA & (1 << PA4)) && (directie != SUS))  directie = JOS;
                if (!(PINA & (1 << PA1)) && (directie != DRT))  directie = STG;
                if (!(PINA & (1 << PA2)) && (directie != STG))  directie = DRT;
		
	
                if (!(PINA & (1 << PA0))) { //resetez, revin la meniu
			
                    mode();
                    continue;
                }
		
			
                deplasare(directie);
                verifica();
                desenMap();
                SnakeDelay();
                lcd_update();
                    
            }
            game_over();
    }
}




